#include <stdio.h>
#include <math.h>
#include <complex.h>


// скалярные типы данных C:

//---------------------------------------------------------

void test01( void ) {      // различные представления
   int i = 123;
   short is = 12;
   char ic1 = 3, ic2 = '2';
   long il = 5932;
   long long ill = 123456; 
   printf( "одни и те же данные могут представляться по-разному:\n" ); 
   printf( " %d \n %d \n %d \n %d \n %lX \n %llX \n",
              i,    is,   ic1,  ic2,  il,    ill );
   printf( "%c => %i\n", ic2, ic2 );
}

//---------------------------------------------------------

const long double PI = 3.141592653589793238462643383279502884197169399375105820974944592307816406\
2862089986280348253421170679821480865132823066470938446095505822317253594081284811174502841027019\
38521105559644622948954930381964428810975665933446128475648233786783165;

void test020( void ) {      // "неточность" вещественных данных
   printf( "все вещественные типы - это только приближённые данные:\n" ); 
   double rd = (double)PI;
   float rf = rd;  
#define FMT "%15.12f"
   printf( FMT" - "FMT" = "FMT"\n", rd, rf, rd - rf );
#undef FMT
}

//---------------------------------------------------------
#define DIV 10.

void test021( void ) {    // максимальная точность итерационных вычислений
   printf( "максимальная точность итерационных вычислений:\n" ); 
   int i;
   float x = 1.;
   double y = 1.;
   for( i = 0; ; i++ ) { 
      if( (float)1. + x == (float)1. + x / (float)DIV ) break;
      x /= (float)DIV; 
   }
   printf( "для float\t: %e (число итераций %d)\n", x, i );
   for( i = 0; ; i++ ) { 
      if( ( (double)1. + y ) == ( (double)1. + y / (double)DIV ) ) break;
      y /= (double)DIV; 
   }
   printf( "для double\t: %e (число итераций %d)\n", x, i );
}

//---------------------------------------------------------

void test022( void ) {    // зависимость от порядка записи вычислений
   printf( "результат зависит от порядка записи вычислений:\n" ); 
   int i;
   double x;
   for( i = 0, x = 1.; ; i++ ) { 
      if( (double)1. + x == (double)1. + x / (double)DIV ) break;
      x /= (double)DIV; 
   }
   printf( "вариант #1\t: %e (число итераций %d)\n", x, i );
   for( i = 0, x = 1.; ; i++ ) { 
      double y1 = x + 1., y2 = ( x /= (double)DIV ) + 1.;
      if( y1 == y2 ) break;    
   }
   printf( "вариант #2\t: %e (число итераций %d)\n", x, i );
}

//---------------------------------------------------------

void test03( void ) {      // перечисления - константы 
   enum months { JAN = 1, FEB, MAR, APR, MAY, JUN,
                 JUL, AUG, SEP, OCT, NOV, DEC };
   int days[] = { 31, 28, 31, 30, 31, 30, 
                  31, 31, 30, 31, 30, 31 }, i;
   printf( "enum - это удобный препроцессорный обман:\n" ); 
   for( i = JAN; i <= DEC; i++ )
      printf( "%d:%d%s", i, days[ i - 1 ], i != DEC ? " , " : "\n" );
}

//---------------------------------------------------------

void test041( void ) {    // длинные вещественные C99
   long double Xld = PI;
   double Xd = (double)PI;
   float Xf = (float)PI;
   printf( "типы данных ISO C99:\n" ); 
   printf( "- long double,\tразмер %d\t: %15.12Le - %15.12Le -> %15.12Le\n",
           sizeof( Xld ), Xld, PI, Xld - PI );
   printf( "- double     ,\tразмер %d\t: %15.12e - %15.12Le -> %15.12e\n",
           sizeof( Xld ), Xd, PI, (float)( (long double)Xd - PI ) );
   printf( "- float      ,\tразмер %d\t: %15.12e - %15.12Le -> %15.12e\n",
           sizeof( Xf ), Xf, PI, (float)( (long double)Xf - PI ) );
}

//---------------------------------------------------------

void test042( void ) {    // комплексные C99
   double complex zc = 1. + 1. * I;
   printf( "типы данных ISO C99:\n" ); 
   printf( "комплексное число: %f %c %fi\n",
           creal( zc ), cimag( zc ) >= 0 ? '+' : '-', cimag( zc ) );
   long double ab = cabsl( zc );
   printf( "модуль этого числа: %Lf\n", ab ); 
}

//---------------------------------------------------------

void ( *tests[] )( void ) = {          // последовательность тестов
        test01,                        // представления данных
        test020, test021, test022,     // вещественные типы 
        test03,                        // константы перечисления
        test041, test042               // новые типы ISO C99
                                       // , test05, test06, ...
};
#include "main.c"

