#include <stdio.h>
#include <string.h>
#include <wchar.h>
#include <complex.h>
#include <math.h>

// агрегаты данных:

//-----------------------------------------------------

void test01( void ) {     // массив
   int ai[] = { 1, 2, 3, 4, 5 }, ni, j;
   printf( "%d элементов массива => ", ni = sizeof( ai ) / sizeof( ai[ 0 ] ) );
   for( j = 0; j < ni; j++ ) 
      printf( "%d%s", ai[ j ], j != ni - 1 ? ", " : "\n" );
}

//-----------------------------------------------------

void test02( void ) {     // разница в представлении
   int ni, j;
   char ac[] = { '1', '2', '3', '4', '5', 0, '6', '\0' };
   printf( "%d элементов массива (но строка длины %d) => ", 
           ni = sizeof( ac ) / sizeof( ac[ 0 ] ), strlen( ac ) );
   for( j = 0; j < ni; j++ ) printf( "%d, ", ac[ j ] );
   printf( "\n" );
}

//-----------------------------------------------------

struct ss {
   int a1, a2, a3;
};

char *put_ss( struct ss *pss ) {
   static char msg[ 40 ];
   sprintf( msg, "%d %d %d", pss->a1, pss->a2, pss->a3 );
   return msg;
}

void test03( void ) {
   struct ss                        // присвоение эквивалентных структурных данных
      vss1 = { 1, 2, 3 }, vss2 = { 3, 2, 1 };
   printf( "присвоение больших эквивалентных структур данных:\n" );
   printf( "было { %s }", put_ss( &vss1 ) );
   vss1 = vss2;
   printf( " ... стало { %s }\n", put_ss( &vss1 ) );
}

//-----------------------------------------------------

void test04( void ) {
   typedef struct ss ss2;
   struct ss vss1 = { 1, 2, 3 };
   ss2 vss2 = { 3, 2, 1 };
   printf( "именная типизация (но не структурная):\n" );
   printf( "было { %s }", put_ss( &vss1 ) );
   *(&vss1) = *(struct ss*)&vss2;   // недопустимо без преобразования типа
   printf( " ... стало { %s }\n", put_ss( &vss1 ) );
}

//-----------------------------------------------------

void test05( void ) {
   int j;
   struct su {            // наложение разнородных данных
      union {
         char c[ 12 ];     
         short s[ 6 ];
         float f[ 3 ];
      };
   } su1 = {{{ '1', '2', '3', '4', '5', '6', 0, 0, 0, 0, 0, 0 }}};

   printf( "наложение разнородных данных:\n" );
   printf( "размер структуры %d байт\n", sizeof( su1 ) );
   printf( "как символы: " );
   for( j = 0; j < sizeof( su1.c ) / sizeof( su1.c[ 0 ] ); j++ )
      if( su1.c[ j ] != 0 ) printf( "%c, ", su1.c[ j ] );
      else printf( "\'\\0\', " );
   printf( "\nкак целые 8-бит: " );
   for( j = 0; j < sizeof( su1.c ) / sizeof( su1.c[ 0 ] ); j++ )
      printf( "%d, ", su1.c[ j ] );
   printf( "\nкак целые 16-бит: " );
   for( j = 0; j < sizeof( su1.s ) / sizeof( su1.s[ 0 ] ); j++ )
      printf( "%d, ", su1.s[ j ] );
   printf( "\nкак вещественные: " );
   for( j = 0; j < sizeof( su1.f ) / sizeof( su1.f[ 0 ] ); j++ )
      printf( "%f, ", su1.f[ j ] );
   printf( "\n" );
   char *p = (char*)&su1;
   printf( "адрес=%p : как строка %s, длиной %d символов\n",
           p, p, strlen( p ) );
}

//-----------------------------------------------------

void test06( void ) {
   struct list {             // массив структур
      char *s;
      int i;
      struct list *next;
   } a[] = {                 // кольцевой связный список
      { "abcd", 1, a + 1 },
      { "efgh", 2, a + 2 },
      { "ijkl", 3, a }
   }, *head;
   int ni = sizeof( a ) / sizeof( a[ 0 ] ), j;
   printf( "массив из %d структур => ", ni );
   for( j = 0; j < ni; j++ ) 
      printf( "%s, ", a[ j ].s );
   printf( "\nсвязный список из %d структур => ", ni );
   for( j = 0, head = a; j < ni; j++, head = head->next ) 
      printf( "%s, ", a[ j ].s );
   printf( "\nкольцевой список => " );
   for( j = 0, head = a; j < ni * 2; j++, head = head->next ) 
      printf( j != ni * 2 - 1 ? "%s, " : "%s ... ...\n", head->s );
}

//-----------------------------------------------------

complex float sqrt_roots1( float A, float B, double C ) {
   float sq = (float)sqrt( B * B - 4. * A * C ),   // решение вещественного квадратного уравнения
         r1 = ( -B + sq ) / 2. / A,
         r2 = ( -B - sq ) / 2. / A;
   return r1 + I * r2;
}

complex float sqrt_roots2( float A, float B, double C ) {
   float sq = (float)sqrt( B * B - 4. * A * C ),   // решение вещественного квадратного уравнения
         r1 = - ( B + ( B >= 0 ? 1. : -1. ) * sq ) / 2. / A,
         r2 = C / A / r1;
   return r1 + I * r2;
}

void test07( void ) {    // искусственное использование типа - C99
   float  b = 2.0, eps = 3.e-10;
   printf( "квадратное уравнение: ( x - %f ) ** 2 = %e\n", b, eps );
   complex float ( *method[] )( float, float, double ) = {
      sqrt_roots1, sqrt_roots2
   };
   int i;
   for( i = 0; i < sizeof( method ) / sizeof( method[ 0 ] ); i++ ) {
      complex float r = method[ i ]( 1., ( -2. * b ), ( b * b - eps ) ); 
      double r1 = creal( r ), r2 = cimag( r ),
             c1 = ( r1 - b ) * ( r1 - b ),
             c2 = ( r2 - b ) * ( r2 - b );
      printf( "корни: %.7f и %.7f"
              " , ( x - 2 ) ** 2 -> %.5e и %.5e\n",
              r1, r2, c1, c2 );
   }
}

//---------------------------------------------------------


void ( *tests[] )( void ) = {  // последовательность тестов
        test01, test02, test03, test04,
        test05, test06, test07
};
#include "main.c"

